SystemOrganization addCategory: #'DAO-Satellite'!

TestCase subclass: #SatelliteSubsystemTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:14'!
testWhenCreatingASubsystemItCanAnswerBasicCommands

	| subsystem |

	"setup"
	subsystem := SatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self assert: (subsystem understands: 'name').
	self assert: (subsystem understands: 'telemetry').
	self assert: (subsystem understands: 'short_telemetry').
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:15'!
testWhenCreatingASubsystemItDoesntUnderstandUnknwonCommand

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self deny: (subsystem understands: 'fruit').
! !

!SatelliteSubsystemTest methodsFor: 'subsystem' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:15'!
testWhenCreatingASubsystemTheIdAndNameIsSet

	| subsystem |

	"setup"
	subsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.

	"verify"	
	self assert: subsystem id equals: 1.
	self assert: subsystem name equals: 'MEMORY'.
! !

TestCase subclass: #SatelliteTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:36'!
satelliteWithThermalAndMemorySubsystems
	| satellite subsystems thermalSubsystem memorySubsystem |

	subsystems := Set new.
	
	memorySubsystem := MemorySatelliteSubsystem newWithId: 1 andName: 'MEMORY'.
	memorySubsystem set: 'free' with: 100.
	subsystems add: memorySubsystem.
	
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem set: 'battery_temp' with: 10.
	thermalSubsystem set: 'radio_temp' with: 20.
	subsystems add: thermalSubsystem.

	satellite := Satellite newWithSubsystems: subsystems.
	^ satellite! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:06'!
testGivenASatelliteContainingSubsystemsAnExistingSubsystemCanBeAccessedByItsId
	| satellite subsystem |
	
	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.
	
	"exercise"
	"subsystem 3 is THERMAL"
	subsystem := satellite subsystemWithId: 3.
	
	"verify"
	self assert: subsystem name equals: 'THERMAL'.! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:04'!
testGivenASatelliteWhenEmittingAShortBeaconItReturnsAShortBeacon

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem set: 'battery_temp' with: '10'.
	thermalSubsystem set: 'radio_temp' with: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"short_telemetry":[{"subsystem_id":"3","battery_temp":"10"}]}'.

	"exercise"	
	beacon := satellite emitShortBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:05'!
testGivenASatelliteWithThermalSubsystemOnlyWhenEmittingABeaconTheBeaconContainsOnlyThermalTelemetry

	| satellite subsystems thermalSubsystem expectedBeacon beacon |

	"setup"
	thermalSubsystem := ThermalSatelliteSubsystem newWithId: 3 andName: 'THERMAL'.
	thermalSubsystem set: 'battery_temp' with: '10'.
	thermalSubsystem set: 'radio_temp' with: '20'.
	subsystems := Set new.
	subsystems add: thermalSubsystem.
	satellite := Satellite newWithSubsystems: subsystems.
	expectedBeacon := '{"telemetry":[{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:26'!
testGivenASatelliteWithTwoSubsystemsWhenEmittingABeaconTheBeaconContainsTwoTelemetryData

	| satellite expectedBeacon beacon |

	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.

	expectedBeacon := '{"telemetry":[{"subsystem_id":"1","free":"100"},{"subsystem_id":"3","battery_temp":"10","radio_temp":"20"}]}'.

	"exercise"	
	beacon := satellite emitBeacon.

	"verify"	
	self assert: beacon equals: expectedBeacon.
! !

!SatelliteTest methodsFor: 'beacon' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:54'!
testWhenSendingCommandToSubsystemIndexedByIdTheSpecificSubsystemResponds
	| satellite response |
	
	"setup"
	satellite := self satelliteWithThermalAndMemorySubsystems.
	
	"exercise"
	"subsystem 3 is THERMAL"
	response := satellite onSubsystem: 3 execute: 'name'.
	
	"verify"
	self assert: response equals: 'THERMAL'.! !

Object subclass: #Satellite
	instanceVariableNames: 'subsystems subsystemsIndexedById'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!Satellite class methodsFor: 'instance creation' stamp: 'EmilianoMenendez 9/7/2013 16:27'!
newWithSubsystems: theSubsystems 
	^ self new initalizeWithSubsystems: theSubsystems! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:48'!
emitBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem telemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'signaling' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:48'!
emitShortBeacon
	| result outputStream telemetryDataOrderedBySubsystemId |
	
	outputStream := WriteStream on: String new.
	
	outputStream nextPutAll: '{"short_telemetry":['.
	
	telemetryDataOrderedBySubsystemId := OrderedIdentityDictionary new.
	
	subsystems do: [ :subsystem | telemetryDataOrderedBySubsystemId at: (subsystem id) put: subsystem telemetry].
	
	telemetryDataOrderedBySubsystemId do: [ :telemetry | outputStream nextPutAll: telemetry. outputStream nextPutAll: ',' ].
	
	result := outputStream contents.
	
	result := result truncateTo: ((result size) - 1).
	
	result := result , ']}'.
	
	^ result.
! !

!Satellite methodsFor: 'initialization' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:08'!
initalizeWithSubsystems: theSubsystems

	subsystems := theSubsystems.
	
	subsystemsIndexedById := Dictionary new.
	
	subsystems do: [ :subsystem | subsystemsIndexedById at: subsystem id put: subsystem ].
! !

!Satellite methodsFor: 'command' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:33'!
onSubsystem: subsystemId execute: commandName
	| response subsystem command |
	
	subsystem := subsystemsIndexedById at: subsystemId.
	
	command := subsystem commands at: commandName.
	
	response := command executeOn: subsystem.
	
	^ response
! !

!Satellite methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:06'!
subsystemWithId: theId

	^ subsystemsIndexedById at: theId
! !

!Satellite methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 19:59'!
subsystems
	
	^ subsystems
! !

Object subclass: #SatelliteCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteCommand subclass: #NameCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!NameCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:30'!
executeOn: target
	^ target name! !

!SatelliteCommand methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:31'!
executeOn: target
	^ self subclassResponsibility.! !

SatelliteCommand subclass: #ShortTelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteCommand subclass: #TelemetryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

Object subclass: #SatelliteSubsystem
	instanceVariableNames: 'id data telemetryKeys shortTelemetryKeys name commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

SatelliteSubsystem subclass: #MemorySatelliteSubsystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!MemorySatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:00'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'free'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'free'.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys.
	
	^ subsystem.! !

!SatelliteSubsystem class methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:28'!
basicCommands
	| commands |
	
	commands := Dictionary new.

	commands at: 'name' put: NameCommand new.
	commands at: 'telemetry' put: TelemetryCommand new.
	commands at: 'short_telemetry' put: ShortTelemetryCommand new.

	^ commands! !

!SatelliteSubsystem class methodsFor: 'creation' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:58'!
newWithId: theId andName: theName
	^ self new initializeWithId: theId andName: theName andTelemetryKeys: Set new andShortTelemetryKeys: Set new! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:27'!
commands
	^ commands! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:10'!
id
	^ id! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 20:26'!
initializeWithId: theId andName: theName andTelemetryKeys: theTelemetryKeys andShortTelemetryKeys: theShortTelemetryKeys
	id := theId.
	name := theName.
	commands := SatelliteSubsystem basicCommands.
	telemetryKeys := theTelemetryKeys.
	shortTelemetryKeys := theShortTelemetryKeys.
	data := Dictionary new.! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:11'!
name
	^ name! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendez 9/9/2013 10:15'!
set: aKey with: aValue 
	data at: aKey put: aValue! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendez 9/9/2013 10:26'!
shortTelemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | (shortTelemetryKeys includes: (entry key)) ifTrue: [temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.]].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendez 9/8/2013 20:40'!
telemetry
	| result temp |
	
	temp := WriteStream on: String new.
	
	temp nextPutAll: '{'.
	
	"add subsytem_id"
	temp nextPutAll: '"subsystem_id":"'.
	temp nextPutAll: id asString.
	temp nextPutAll: '",'.
	
	"build key:value,key:value,,, string "
	data associationsDo: [ :entry | temp nextPutAll: '"'. temp nextPutAll: (entry key asString) . temp nextPutAll: '":"'. temp nextPutAll: (entry value asString). temp nextPutAll: '",'.].
	
	"send stream to string"
	result := temp contents.
	
	"remove last ,"
	result := result truncateTo: ((result size) - 1).
	
	result := result , '}'.
	
	^ result
! !

!SatelliteSubsystem methodsFor: 'accessing' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 18:57'!
understands: aCommandName 
	^ (commands includes: aCommandName).! !

SatelliteSubsystem subclass: #ThermalSatelliteSubsystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DAO-Satellite'!

!ThermalSatelliteSubsystem class methodsFor: 'as yet unclassified' stamp: 'EmilianoMenendezEzequielDavidovichCaballero 9/9/2013 16:01'!
newWithId: theId andName: theName
	| subsystem telemetryKeys shortTelemetryKeys |
	
	telemetryKeys := Set new.
	telemetryKeys add: 'battery_temp'.
	telemetryKeys add: 'radio_temp'.
	
	shortTelemetryKeys := Set new.
	shortTelemetryKeys add: 'battery_temp'.
	
	subsystem := self new initializeWithId: theId andName: theName andTelemetryKeys: telemetryKeys andShortTelemetryKeys: shortTelemetryKeys.
	
	^ subsystem.! !
